@@ -25,19 +25,6 @@ from dvc.utils.http import open_url
 
 logger = logging.getLogger(__name__)
 
-STATUS_OK = 1
-STATUS_MISSING = 2
-STATUS_NEW = 3
-STATUS_DELETED = 4
-
-STATUS_MAP = {
-    # (local_exists, remote_exists)
-    (True, True): STATUS_OK,
-    (False, False): STATUS_MISSING,
-    (True, False): STATUS_NEW,
-    (False, True): STATUS_DELETED,
-}
-
 
 class RemoteCmdError(DvcException):
     def __init__(self, remote, cmd, ret, err):
@@ -57,7 +44,7 @@ class RemoteMissingDepsError(DvcException):
     pass
 
 
-class BaseRemoteTree:
+class BaseTree:
     scheme = "base"
     REQUIRES = {}
     PATH_CLS = URLInfo
@@ -169,7 +156,7 @@ class BaseRemoteTree:
 
         raise RemoteActionNotImplemented("open", self.scheme)
 
-    def exists(self, path_info):
+    def exists(self, path_info, use_dvcignore=True):
         raise NotImplementedError
 
     # pylint: disable=unused-argument
@@ -249,54 +236,51 @@ class BaseRemoteTree:
             return False
         return hash_.endswith(cls.CHECKSUM_DIR_SUFFIX)
 
-    def get_hash(self, path_info, tree=None, **kwargs):
+    def get_hash(self, path_info, **kwargs):
         assert path_info and (
             isinstance(path_info, str) or path_info.scheme == self.scheme
         )
 
-        if not tree:
-            tree = self
+        if not self.exists(path_info):
+            return self.PARAM_CHECKSUM, None
 
-        if not tree.exists(path_info):
-            return None
+        # pylint: disable=assignment-from-none
+        hash_ = self.state.get(path_info)
 
-        if tree == self:
-            # pylint: disable=assignment-from-none
-            hash_ = self.state.get(path_info)
-        else:
-            hash_ = None
         # If we have dir hash in state db, but dir cache file is lost,
         # then we need to recollect the dir via .get_dir_hash() call below,
         # see https://github.com/iterative/dvc/issues/2219 for context
         if (
             hash_
             and self.is_dir_hash(hash_)
-            and not tree.exists(self.cache.tree.hash_to_path_info(hash_))
+            and not self.cache.tree.exists(
+                self.cache.tree.hash_to_path_info(hash_)
+            )
         ):
             hash_ = None
 
         if hash_:
-            return hash_
+            return self.PARAM_CHECKSUM, hash_
 
-        if tree.isdir(path_info):
-            hash_ = self.get_dir_hash(path_info, tree, **kwargs)
+        if self.isdir(path_info):
+            typ, hash_ = self.get_dir_hash(path_info, **kwargs)
         else:
-            hash_ = tree.get_file_hash(path_info)
+            typ, hash_ = self.get_file_hash(path_info)
 
         if hash_ and self.exists(path_info):
             self.state.save(path_info, hash_)
 
-        return hash_
+        return typ, hash_
 
     def get_file_hash(self, path_info):
         raise NotImplementedError
 
-    def get_dir_hash(self, path_info, tree, **kwargs):
+    def get_dir_hash(self, path_info, **kwargs):
         if not self.cache:
             raise RemoteCacheRequiredError(path_info)
 
-        dir_info = self._collect_dir(path_info, tree, **kwargs)
-        return self._save_dir_info(dir_info, path_info)
+        dir_info = self._collect_dir(path_info, **kwargs)
+        return self.save_dir_info(dir_info)
 
     def hash_to_path_info(self, hash_):
         return self.path_info / hash_[0:2] / hash_[2:]
@@ -309,29 +293,28 @@ class BaseRemoteTree:
 
         return "".join(parts)
 
-    def save_info(self, path_info, tree=None, **kwargs):
-        return {
-            self.PARAM_CHECKSUM: self.get_hash(path_info, tree=tree, **kwargs)
-        }
+    def save_info(self, path_info, **kwargs):
+        typ, hash_ = self.get_hash(path_info, **kwargs)
+        return {typ: hash_}
 
-    @staticmethod
-    def _calculate_hashes(file_infos, tree):
+    def _calculate_hashes(self, file_infos):
         file_infos = list(file_infos)
         with Tqdm(
             total=len(file_infos),
             unit="md5",
             desc="Computing file/dir hashes (only done once)",
         ) as pbar:
-            worker = pbar.wrap_fn(tree.get_file_hash)
-            with ThreadPoolExecutor(max_workers=tree.hash_jobs) as executor:
-                tasks = executor.map(worker, file_infos)
-                hashes = dict(zip(file_infos, tasks))
-        return hashes
+            worker = pbar.wrap_fn(self.get_file_hash)
+            with ThreadPoolExecutor(max_workers=self.hash_jobs) as executor:
+                hashes = (
+                    value for typ, value in executor.map(worker, file_infos)
+                )
+                return dict(zip(file_infos, hashes))
 
-    def _collect_dir(self, path_info, tree, **kwargs):
+    def _collect_dir(self, path_info, **kwargs):
         file_infos = set()
 
-        for fname in tree.walk_files(path_info, **kwargs):
+        for fname in self.walk_files(path_info, **kwargs):
             if DvcIgnore.DVCIGNORE_FILE == fname.name:
                 raise DvcIgnoreInCollectedDirError(fname.parent)
 
@@ -340,7 +323,7 @@ class BaseRemoteTree:
         hashes = {fi: self.state.get(fi) for fi in file_infos}
         not_in_state = {fi for fi, hash_ in hashes.items() if hash_ is None}
 
-        new_hashes = self._calculate_hashes(not_in_state, tree)
+        new_hashes = self._calculate_hashes(not_in_state)
         hashes.update(new_hashes)
 
         result = [
@@ -362,8 +345,8 @@ class BaseRemoteTree:
         # Sorting the list by path to ensure reproducibility
         return sorted(result, key=itemgetter(self.PARAM_RELPATH))
 
-    def _save_dir_info(self, dir_info, path_info):
-        hash_, tmp_info = self._get_dir_info_hash(dir_info)
+    def save_dir_info(self, dir_info):
+        typ, hash_, tmp_info = self._get_dir_info_hash(dir_info)
         new_info = self.cache.tree.hash_to_path_info(hash_)
         if self.cache.changed_cache_file(hash_):
             self.cache.tree.makedirs(new_info.parent)
@@ -371,13 +354,14 @@ class BaseRemoteTree:
                 tmp_info, new_info, mode=self.cache.CACHE_MODE
             )
 
-        if self.exists(path_info):
-            self.state.save(path_info, hash_)
         self.state.save(new_info, hash_)
 
-        return hash_
+        return typ, hash_
 
     def _get_dir_info_hash(self, dir_info):
+        # Sorting the list by path to ensure reproducibility
+        dir_info = sorted(dir_info, key=itemgetter(self.PARAM_RELPATH))
+
         tmp = tempfile.NamedTemporaryFile(delete=False).name
         with open(tmp, "w+") as fobj:
             json.dump(dir_info, fobj, sort_keys=True)
@@ -387,8 +371,8 @@ class BaseRemoteTree:
         to_info = tree.path_info / tmp_fname("")
         tree.upload(from_info, to_info, no_progress_bar=True)
 
-        hash_ = tree.get_file_hash(to_info) + self.CHECKSUM_DIR_SUFFIX
-        return hash_, to_info
+        typ, hash_ = tree.get_file_hash(to_info)
+        return typ, hash_ + self.CHECKSUM_DIR_SUFFIX, to_info
 
     def upload(self, from_info, to_info, name=None, no_progress_bar=False):
         if not hasattr(self, "_upload"):
